package monalipse.server;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import monalipse.MonalipsePlugin;
import monalipse.server.giko.GikoServer;

import org.apache.commons.httpclient.HttpClient;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.preference.IPreferenceStore;

public class BBSServerManager
{
	public static final String PREF_PROXY_USE = "proxy.use";
	private static final boolean DEFAULT_PROXY_USE = false;
	public static final String PREF_PROXY_HOST = "proxy.host";
	private static final String DEFAULT_PROXY_HOST = "";
	public static final String PREF_PROXY_PORT = "proxy.port";
	private static final int DEFAULT_PROXY_PORT = 8080;

	private static Map instances = new HashMap();
	private static IBBSServer[] servers = new IBBSServer[0];

	static
	{
		IPreferenceStore store = MonalipsePlugin.getDefault().getPreferenceStore();
		store.setDefault(PREF_PROXY_USE, DEFAULT_PROXY_USE);
		store.setDefault(PREF_PROXY_HOST, DEFAULT_PROXY_HOST);
		store.setDefault(PREF_PROXY_PORT, DEFAULT_PROXY_PORT);
	}
	
	private static void ensureProjectRetarget()
	{
		if(servers.length != 0)
			return;
		
		IProject project = MonalipsePlugin.getProject();
		if(project != null)
			ensureProjectRetarget(project);
	}
	
	private static void ensureProjectRetarget(IProject project)
	{
		List list = new ArrayList();
		IBBSServer server;

		server = new GikoServer();
		server.initialize(project.getFolder(server.getTargetName()));
		list.add(server);

		servers = new IBBSServer[list.size()];
		list.toArray(servers);
	}
	
	public static IThreadContentProvider getThreadContentProviderOf(URL url)
	{
		IBBSBoard board = getBoardOf(url);
		if(board == null)
			return null;
		else
			return board.getThreadOf(url);
	}
	
	public static IBBSBoard getBoardOf(URL url)
	{
		ensureProjectRetarget();
		for(int i = 0; i < servers.length; i++)
		{
			IBBSBoard board = servers[i].getBoardOf(url);
			if(board != null)
				return board;
		}
		return null;
	}
	
	public static URL getURLOf(IFile file)
	{
		ensureProjectRetarget();
		for(int i = 0; i < servers.length; i++)
		{
			URL url = servers[i].getURLOf(file);
			if(url != null)
				return url;
		}
		return null;
	}

	public static IBBSServer getInstanceOf(String target)
	{
		ensureProjectRetarget();
		for(int i = 0; i < servers.length; i++)
		{
			if(servers[i].getTargetName().equals(target))
				return servers[i];
		}
		return null;
	}

	public static void configureHttpClient(HttpClient hc, URL url)
	{
		int port = url.getPort();
		if(port == -1)
			port = url.getDefaultPort();
		hc.getHostConfiguration().setHost(url.getHost(), port, url.getProtocol());
		IPreferenceStore store = MonalipsePlugin.getDefault().getPreferenceStore();
		if(store.getBoolean(BBSServerManager.PREF_PROXY_USE))
			hc.getHostConfiguration().setProxy(store.getString(BBSServerManager.PREF_PROXY_HOST), store.getInt(BBSServerManager.PREF_PROXY_PORT));
	}
}
